function [OutCellArray,Outconfig]=AverageDatasets(InCellArray, Inconfig)
% AVERAGEDATASETS computes a pixel-by-pixel average of many maps, masks, or freqmaps.
%
% Given multiple datasets of the same type, it is sometimes useful to
% average them all together. For instance, there may be multiple
% measurements taken over several days. This routine will combine several
% datasets into a single dataset by taking the pixel-by-pixel average.
%
% Note: All datasets must have the same properties.
%
% Inputs: two or more data sets of the same type (freqmap, map or mask)
% Outputs: one data type of the same type as the input (freqmap, map or mask)
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 2012/10/8 - Version 1.0 - Gerg Smith - initial implementation

% default values (in case something happens) will be the first data set
Outconfig=Inconfig;
OutCellArray{1}=InCellArray{1};

% Process each and every data set that is passed in. Because the number of
% data sets is not known ahead of time, they are processed in a 'for' loop.
% In this case, a running total is computed.
dataSum = InCellArray{1}.data;
askAgain = Inconfig(strcmpi(strtrim(Inconfig), 'checkHeader'),2);
if ~isempty(askAgain), askAgain = str2num(askAgain{1}); else askAgain = false; end %#ok<ST2NM> 2014-07-09 G.S. - str2num needed because value is boolean
for dataCount = 2:length(InCellArray)
    if ~CompareDataSize(InCellArray{dataCount},InCellArray{1})
        % Data arrays are of different sizes. Abort!
        warndlg(['Datasets ''',InCellArray{dataCount}.name,''' and ''',InCellArray{1}.name,''' have incompatible array sizes. ',...
            'Aborting calculation.'],'Incompatible Arrays','modal');
        OutCellArray = {};
        return;
    elseif ~CompareHeaders(InCellArray{dataCount},InCellArray{1}) && askAgain
        % Something in the header is different from the first header
        % ask the user to abort or continue
        choice = questdlg({['Datasets ''',InCellArray{dataCount}.name,''' and ''',InCellArray{1}.name,''' have different properties. ',...
            'Continuing calculation may produce incorrect results.'],'Do you wish to continue calculating?'}, ...
         'Different datasets!', ...
         'Continue','Cancel','Don''t ask again','Continue');
        % Handle response
        switch choice
            case 'Cancel'
                % user aborted calculation
                OutCellArray = {};
                return;
            case 'Don''t ask again'
                % stop pestering the user
                askAgain = false;
        end
    end
    currentData = InCellArray{dataCount}.data;
    dataSum = dataSum + currentData;
end

% the average is the running total divided by the number of elements
dataAvg = dataSum/length(InCellArray);

% set output data
% note: headers implicitly equal to headers in first dataset
OutCellArray{1}.data = dataAvg;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  SUBROUTINES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function agree = CompareHeaders(cellArray1, cellArray2)
% compare the header information from two cell arrays.
    
    % get header information
    % exclude items expected to change (i.e. creation date)
    header1 = strtrim(cellArray1.header(2:end));
    header2 = strtrim(cellArray2.header(2:end));
    
    % everything is a string, so compare them item-by-item
    % agreement only occurs if all strings are the same
    comparison = strcmpi(header1,header2);
    agree = all(comparison);
end

function agree = CompareDataSize(cellArray1, cellArray2)
% compare the array size for two data sets
    
    % get data size for each cell array
    size1 = size(cellArray1.data);
    size2 = size(cellArray2.data);
    
    % compare sizes
    agree = all(size1==size2);
end
