function [OutCellArray, Outconfig] = StructureFunction(InCellArray, Inconfig)
%
% This module calculates the structure function, the rms difference of
% values in map as function of separation.  Structure functions statistically
% define phase differences for pairs of points at different spatial separation.
% Please note that if the map is error in surface, it has to be multiplied
% by 2 to computer the rms wavefront difference. This code assumes square pixels.
%
% Skip Factor: interval between rows and columns used as first points of pairs in pixels.
% In other words, the first points of the pairs are on a downsampled grid.
%
% nSeps: number of separations.
%
% maxSepRatio: the maximum separation in ratio.  If matrix is not square, it
% is the ratio of the maximum sepration to the long side of the matrix.
%
% logSampled: 1 for equally spaced in log, and 0 for equally spaced in linear space
%
%Inputs: Map
%Outputs: Coordinates
%
%Author: Ping Zhou
%Email: pzhou@optics.arizona.edu
%
% Version history:
% 11/19/12 - version 1
%
Outconfig=Inconfig;

map = InCellArray{1}.data;
mask = isfinite(map);

xsize=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2};
ysize=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2};

xunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Unit'),2};
yunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Unit'),2};
zunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Z Unit'),2};

validinputs=0;
skipFactor = Inconfig{strcmpi(Inconfig, 'skipFactor(pixel)'),2};
nSeps = Inconfig{strcmpi(Inconfig, 'nSeps'),2};
maxSepRatio = Inconfig{strcmpi(Inconfig, 'maxSepRatio'),2};
logSampled = Inconfig{strcmpi(Inconfig, 'logSampled'),2};

while ~validinputs
    
    prompt = {'skip factor (interval between rows and columns used as first points of pairs in pixels):',...
        'number of separations in the plot:',...
        'max separation (ratio):',...
        'linear spaced separation (0) or  logarithmically spaced separation (1)'};
    dlg_title = 'Input parameters';
    num_lines = 1;
    answer = inputdlg(prompt, dlg_title, num_lines, {num2str(skipFactor), num2str(nSeps),num2str(maxSepRatio),num2str(logSampled)});
    
    if ~isempty(answer)
        tempSkipFactor = str2num(answer{1});
        tempnSeps = str2num(answer{2});
        tempMaxSepRatio = str2num(answer{3});
        tempLogSampled = str2num(answer{4});
        
        if ((~isempty(tempSkipFactor))&& (~isempty(tempnSeps))&&(~isempty(tempMaxSepRatio))&&(~isempty(tempLogSampled)))
            validinputs=1;
            skipFactor=tempSkipFactor;
            nSeps = tempnSeps;
            maxSepRatio = tempMaxSepRatio;
            logSampled = tempLogSampled;
        else
            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

Outconfig{strcmpi(Outconfig, 'skipFactor(pixel)'),2}=num2str(skipFactor);
Outconfig{strcmpi(Outconfig, 'nSeps'),2}=num2str(nSeps);
Outconfig{strcmpi(Outconfig, 'maxSepRatio'),2}=num2str(maxSepRatio);
Outconfig{strcmpi(Outconfig, 'logSampled'),2}=num2str(logSampled);


[rows,cols] = size(map);
pixSize = str2num(xsize)/cols;
minSep = pixSize;
maxSep = max(str2num(xsize),str2num(ysize))*maxSepRatio;
binSize = minSep;

if (logSampled)
    seps = logspace(log10(minSep),log10(maxSep),nSeps);
else
    seps = linspace(minSep, maxSep, nSeps);
end

binSizes = binSize*ones(1,nSeps);

pixSeps = seps/pixSize;
pixBinSizes = binSizes/pixSize;
minPixSeps = pixSeps - pixBinSizes/2;
maxPixSeps = pixSeps + pixBinSizes/2;
sumsq = zeros(1,nSeps);
nPairs = zeros(1,nSeps);
firstPoints = 0;

%
% For each valid first point...
%
for i1=1:skipFactor:rows
    for j1=1:skipFactor:cols
        if mask(i1,j1)
            firstPoints = firstPoints + 1;
            %
            % ...for each valid second point...
            %
            for i2=1:rows
                for j2=1:cols
                    if mask(i2,j2)
                        %
                        % ...evaluate separation in pixels.
                        %
                        pixSep = sqrt((i2-i1)^2+(j2-j1)^2);
                        %
                        % ...increment sum of squares of difference and
                        % number of samples in appropriate separation bin.
                        %
                        for k=1:nSeps
                            if pixSep > minPixSeps(k) && pixSep < maxPixSeps(k)
                                sumsq(k) = sumsq(k) + (map(i1,j1) - map(i2,j2))^2;
                                nPairs(k) = nPairs(k) + 1;
                                continue % No need to check other bins.
                            end
                        end
                    end
                end
            end
        end
    end
end
rmsDif = sqrt(sumsq ./ nPairs);

% display
figure
if (logSampled)
    loglog(seps,rmsDif,'-ok');
else
    plot(seps,rmsDif,'-ok');
end
xlabel(['separation (', [xunit],')'])
ylabel(['rms difference (' [zunit],')'])
grid on

OutCellArray{1}.data=[seps; rmsDif]';
OutCellArray{1}.type='coordinates';

OutCellArray{1}.header={'Date Created', ''; 'Column Unit', [xunit,', ',zunit]; 'Column Label', 'seps, rmsDif'};

