function [OutCellArray,Outconfig]=TrimData(InCellArray, Inconfig)
%TrimData removes excess NaN values from a rectangular dataset.
%
% The goal of this module is to remove excess NaN values surrounding a data
% set. When 'Trim Aggressive' is set to false, then the data will be
% removed symmetrically from opposite sides. In other words, the center of
% the data will remain the center of the data at the cost of extra
% rows/columns of empty data. When 'Trim Aggressive' is true, the original
% center pixel may not be the same as the final center pixel, but all empty
% rows/columns will be trimmed. Additional parameter 'MakeSquare' will pad
% the smallest dimension of the trimmed map to make the output a square
% array.
%
% One application of this routine is after masking data, such as by a
% threshold. Typically this produces maps which have extra NaN values. This
% padding can affect other functions such as Zernike calculations and
% therefore can be removed with this module.
%
% Inputs: Map, Freqmap
% Outputs: Same as Input
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 2012/05/24 - added Make Square option and improved dimension handling
% 2012/05/22 - initial implementation

    Outconfig=Inconfig;
    OutCellArray{1}=InCellArray{1};

    
    % get map or freqmap from the input
    dataMap = InCellArray{1}.data;
    trimAggressive = str2num(Inconfig{strcmp(strtrim(Inconfig), 'Trim Aggressive'),2});  %#ok - G.S. - 2012/05/22 - str2num is required over str2double since the entered value could be a number (0 or 1), or a string ('true' or 'false')
    makeSquare = str2num(Inconfig{strcmp(strtrim(Inconfig), 'Make Square'),2});          %#ok - G.S. - 2012/05/22 - str2num is required over str2double since the entered value could be a number (0 or 1), or a string ('true' or 'false')
    
    % query user about agressiveness
    choice = questdlg(...
        {['Some routines require the center pixel to remain unchanged. ',...
            'Others can accept more aggressive removal of NaNs. ',...
            '''Keep Data Centered'' may leave some empty rows and/or columns but ',...
            'will not change the center of your data set. '],...
            'Should the trimmed data remain centered?'}, ...
        'Should data remain centered?', ...
        'Keep Data Centered','Remove All NaNs','Keep Data Centered');
    switch choice
        case 'Remove All NaNs'
            trimAggressive = true;
        otherwise
            trimAggressive = false;
    end
    
    % query user about being square
    choice = questdlg(...
        {['Some routines require or work better with square output arrays as opposed to rectangular arrays.',...
        'This routine can make the output square by appending NaNs to the small dimension'],...
            'Should the returned data be in a square array?'}, ...
        'Should data be squared?', ...
        'Make it Square','Rectangle is Fine','Rectangle is Fine');
    switch choice
        case 'Make it Square'
            makeSquare = true;
        otherwise
            makeSquare = false;
    end
    
    
    % function for determining how many rows/columns should be trimmed
    function counterArray = RowsToTrim(data)
        % count rows from beginning
        counter = 1;
        while all(isnan(data(counter,:)))
            counter = counter + 1;
        end
        counter = counter - 1;
        counterArray(1) = counter;
        
        % count rows from end
        counter = 1;
        while all(isnan(data(end-counter-1,:)))
            counter = counter + 1;
        end
        counter = counter - 1;
        counterArray(2) = counter;
    end


    % get number of rows and columns to trim
    if trimAggressive
        trimRows = RowsToTrim(dataMap);
        trimColumns = RowsToTrim(dataMap');
        trimArray = [trimRows(1), trimRows(2), trimColumns(1), trimColumns(2)];
    else
        trimRows = min(RowsToTrim(dataMap));
        trimColumns = min(RowsToTrim(dataMap'));
        trimArray = [trimRows, trimRows, trimColumns, trimColumns];
    end
    
    
    % trim data
    mapTrim = dataMap(trimArray(1)+1:end-trimArray(2), trimArray(3)+1:end-trimArray(4));
    
    % make data square
    if makeSquare
        dataSize = size(mapTrim);
        squareSize = max(dataSize);
        mapCoords = [...
            1 + ceil((squareSize-dataSize(1))/2),...
            squareSize - floor((squareSize-dataSize(1))/2),...
            1 + ceil((squareSize-dataSize(2))/2),...
            squareSize - floor((squareSize-dataSize(2))/2),...
            ];
        mapOut = nan(squareSize,squareSize);
        mapOut(mapCoords(1):mapCoords(2),mapCoords(3):mapCoords(4)) = mapTrim;
    else
        mapOut = mapTrim;
    end
    
    % update data information for return
    OutCellArray{1}.data = mapOut;
    
    % update size information for return
    originalSize = size(dataMap);
    newSize = size(mapOut);
    currentSize = [...
        str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2}),...
        str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2})
        ];
    scaleFactor = newSize./originalSize;
    OutCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2} = num2str(currentSize(1)*scaleFactor(1));
    OutCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2} = num2str(currentSize(2)*scaleFactor(2));
end

